# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: fed_grpc.proto
# Protobuf Python Version: 5.27.2
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    27,
    2,
    '',
    'fed_grpc.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0e\x66\x65\x64_grpc.proto\x12\x04main\"F\n\x0cregisterArgs\x12\n\n\x02ip\x18\x01 \x01(\t\x12\x0c\n\x04port\x18\x02 \x01(\t\x12\x0b\n\x03\x63id\x18\x03 \x01(\t\x12\x0f\n\x07pub_key\x18\x04 \x01(\t\"5\n\x0bregisterOut\x12\x17\n\x0f\x63onnectedClient\x18\x01 \x01(\x08\x12\r\n\x05round\x18\x02 \x01(\x05\"\x1a\n\nsampleSize\x12\x0c\n\x04size\x18\x01 \x01(\x05\"\x17\n\x08\x61\x63\x63uracy\x12\x0b\n\x03\x61\x63\x63\x18\x01 \x01(\x02\"\x1c\n\nweightList\x12\x0e\n\x06weight\x18\x01 \x03(\x02\"\x1d\n\x0c\x63urrentRound\x12\r\n\x05round\x18\x01 \x01(\x05\"\x06\n\x04void\"\x19\n\tChunkData\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\x0c\"\'\n\rEncryptedData\x12\x16\n\x0e\x65ncrypted_data\x18\x01 \x01(\x0c\"\x1b\n\x08Response\x12\x0f\n\x07message\x18\x01 \x01(\t\":\n\x10registerResponse\x12\x17\n\x0f\x63onnectedClient\x18\x01 \x01(\x08\x12\r\n\x05round\x18\x02 \x01(\x05\"<\n\x0e\x45ncryptionArgs\x12*\n\rencryptedData\x18\x01 \x01(\x0b\x32\x13.main.EncryptedData2\xda\x03\n\x10\x46\x65\x64\x65ratedService\x12.\n\tsendChunk\x12\x0f.main.ChunkData\x1a\x0e.main.Response(\x01\x12\x37\n\x0e\x63lientRegister\x12\x12.main.registerArgs\x1a\x11.main.registerOut\x12-\n\rstartLearning\x12\n.main.void\x1a\x10.main.weightList\x12-\n\rgetSampleSize\x12\n.main.void\x1a\x10.main.sampleSize\x12\x33\n\x0fmodelValidation\x12\x10.main.weightList\x1a\x0e.main.accuracy\x12$\n\nkillClient\x12\n.main.void\x1a\n.main.void\x12+\n\tsendRound\x12\x12.main.currentRound\x1a\n.main.void\x12\x39\n\x11sendEncryptedData\x12\x14.main.EncryptionArgs\x1a\x0e.main.Response\x12<\n\x14receiveEncryptedData\x12\x14.main.EncryptionArgs\x1a\x0e.main.Responseb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'fed_grpc_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_REGISTERARGS']._serialized_start=24
  _globals['_REGISTERARGS']._serialized_end=94
  _globals['_REGISTEROUT']._serialized_start=96
  _globals['_REGISTEROUT']._serialized_end=149
  _globals['_SAMPLESIZE']._serialized_start=151
  _globals['_SAMPLESIZE']._serialized_end=177
  _globals['_ACCURACY']._serialized_start=179
  _globals['_ACCURACY']._serialized_end=202
  _globals['_WEIGHTLIST']._serialized_start=204
  _globals['_WEIGHTLIST']._serialized_end=232
  _globals['_CURRENTROUND']._serialized_start=234
  _globals['_CURRENTROUND']._serialized_end=263
  _globals['_VOID']._serialized_start=265
  _globals['_VOID']._serialized_end=271
  _globals['_CHUNKDATA']._serialized_start=273
  _globals['_CHUNKDATA']._serialized_end=298
  _globals['_ENCRYPTEDDATA']._serialized_start=300
  _globals['_ENCRYPTEDDATA']._serialized_end=339
  _globals['_RESPONSE']._serialized_start=341
  _globals['_RESPONSE']._serialized_end=368
  _globals['_REGISTERRESPONSE']._serialized_start=370
  _globals['_REGISTERRESPONSE']._serialized_end=428
  _globals['_ENCRYPTIONARGS']._serialized_start=430
  _globals['_ENCRYPTIONARGS']._serialized_end=490
  _globals['_FEDERATEDSERVICE']._serialized_start=493
  _globals['_FEDERATEDSERVICE']._serialized_end=967
# @@protoc_insertion_point(module_scope)
